@@ -3,7 +3,10 @@ from typing import (
     Optional,
     Tuple,
     Hashable,
-    List
+    TypeVar,
+    Dict,
+    Union,
+    Iterable
 )
 
 
@@ -11,6 +14,11 @@ Symbol = str
 Payload = object
 Vector = Tuple[Hashable, ...]
 
+T = TypeVar('T')
+
+HierarchicalDict = Dict[Hashable, Union[T, 'HierarchicalDict[T]']]
+HierarchicalDictV = Union[T, 'HierarchicalDict[T]']
+
 
 def match_vector(
     vector: Vector,
@@ -36,22 +44,29 @@ def match_vector(
     return vector == sub_target
 
 
-def set_hierachical(
-    target: dict,
+def set_hierarchical(
+    target: HierarchicalDict[T],
     vector: Vector,
-    value,
+    value: T,
     loose: bool,
-    context: List[Hashable] = []
-) -> Tuple[bool, Optional[list]]:
+    context: Iterable[Hashable] = []
+) -> Tuple[bool, Iterable[Hashable]]:
     """Set the value to a dict hirachically
 
     Args:
         vector (tuple): the length of vector must be larger than 0
         loose (bool): If `False`, if the target already exists,
         it will treat it as a failure
+
+    Returns:
+        tuple:
+            - bool: whether the value is set
+            - list: the context of the value
     """
 
     first = vector[0]
+    current_context = [*context, first]
+
 
     if len(vector) == 1:
         # The last item
@@ -59,76 +74,82 @@ def set_hierachical(
             # Which means it is the last item of the vector,
             # we just set the value
             target[first] = value
-            return True, None
+            return True, current_context
         else:
-            return False, [*context, first]
+            return False, current_context
 
     if first in target:
         current = target[first]
 
-        if type(current) is not dict:
+        if not isinstance(current, dict):
             # There is a conflict
-            return False, [*context, first]
+            return False, current_context
     else:
         # The next level does not exists, we just create it
         current = {}
         target[first] = current
 
-    return set_hierachical(
+    return set_hierarchical(
         current,
         vector[1:],
         value,
         loose,
-        [*context, first]
+        current_context
     )
 
 
-def get_hierachical(
-    target: dict,
+def get_hierarchical(
+    target: HierarchicalDict[T],
     vector: Vector
-) -> Any:
-    """Get a property from a nested dict
+) -> Optional[Union[T, HierarchicalDict[T]]]:
+    """Get a property from a hierarchical dict
 
     Args:
         target (dict): the dict
         vector (tuple):
     """
 
+    current: HierarchicalDictV[T] = target
+
     for key in vector:
-        if key not in target:
-            return
+        if not isinstance(current, dict) or key not in current:
+            return None
 
-        target = target[key]
+        current = current[key]
 
-    return target
+    return current
 
 
-def get_partial_hierachical(
-    target: dict,
+def get_partial_hierarchical(
+    target: HierarchicalDict[T],
     vector: Vector
-) -> Any:
-    """Get a property from a nested dict, it will
+) -> Optional[T]:
+    """Get a property from a hierarchical dict, it will
     return the first non-dict object
     """
 
+    current: HierarchicalDictV[T] = target
+
     for key in vector:
-        if key not in target:
-            return
+        if not isinstance(current, dict):
+            return current
 
-        target = target[key]
+        if key not in current:
+            return None
 
-        if type(target) is not dict:
-            return target
+        current = current[key]
+
+    return None if isinstance(current, dict) else current
 
 
 VECTOR_SEPARATOR = ','
 
 
-def stringify_vector(list_like):
+def stringify_vector(list_like: Iterable[Hashable]) -> str:
     return f'<{VECTOR_SEPARATOR.join([str(x) for x in list_like])}>'
 
 
-def stringify(self, name: str) -> str:
+def v_stringify(self, name: str) -> str:
     try:
         vector_str = stringify_vector(self.vector)
     except Exception:
@@ -137,6 +158,18 @@ def stringify(self, name: str) -> str:
     return name + vector_str
 
 
+def vs_stringify(self, name: str) -> str:
+    try:
+        vectors = stringify_vector([
+            stringify_vector(vector)
+            for vector in self.vectors
+        ])
+    except Exception:
+        return f'{name}<invalid>'
+
+    return name + vectors
+
+
 def is_hashable(subject: Any) -> bool:
     try:
         hash(subject)
@@ -145,6 +178,16 @@ def is_hashable(subject: Any) -> bool:
         return False
 
 
+def check_vectors(vectors: Any, target):
+    if not isinstance(vectors, Iterable):
+        raise ValueError(
+            f'vectors of {target} must be an iterable, but got `{vectors}`'
+        )
+
+    for vector in vectors:
+        check_vector(vector, target)
+
+
 def check_vector(vector, target):
     if not isinstance(vector, tuple):
         raise ValueError(
