@@ -2,7 +2,7 @@ import asyncio
 import logging
 from functools import partial
 from typing import (
-    Dict, FrozenSet, Iterable, List, Set
+    Dict, FrozenSet, Iterable, List, Set, Coroutine
 )
 
 from .provider import Provider
@@ -14,11 +14,13 @@ from .consumer import (
 from .reducer import Reducer
 
 from .common import (
+    HierarchicalDict,
+
     match_vector,
     stringify_vector,
-    set_hierachical,
-    get_hierachical,
-    get_partial_hierachical,
+    set_hierarchical,
+    get_hierarchical,
+    get_partial_hierarchical,
 
     Vector,
     Symbol,
@@ -31,13 +33,16 @@ logger = logging.getLogger(__name__)
 
 class Orchestrator:
     """
+    The orchestrator is the core of the compton framework.
     """
 
     MAX_INIT_RETRIES = 3
 
+    _store: HierarchicalDict[Payload]
+    _tasks: Dict[Symbol, asyncio.Task]
     _providers: Dict[Vector, Provider]
-    _subscribed: Dict[Vector, List[Consumer]]
-    _reducers: Iterable[Reducer]
+    _subscribed: Dict[Vector, List[ConsumerSentinel]]
+    _reducers: HierarchicalDict[Reducer]
     _added: Set[Symbol]
 
     def __init__(
@@ -49,7 +54,7 @@ class Orchestrator:
         self._providers = {}
         self._subscribed = {}
         self._reducers = {}
-
+        self._tasks = {}
         self._added = set()
 
         self._apply_reducers(reducers)
@@ -71,6 +76,9 @@ class Orchestrator:
         self.__loop = loop
         return loop
 
+    def _create_task(self, coro: Coroutine) -> asyncio.Task:
+        return self._loop.create_task(coro)
+
     def _apply_reducers(
         self,
         reducers: Iterable[Reducer]
@@ -80,17 +88,18 @@ class Orchestrator:
         for reducer in reducers:
             Reducer.check(reducer)
 
-            vector = reducer.vector
+            vectors = reducer.vectors
 
-            # We set hierachically for reducers, because
-            # we allow reducers to do a semi matching
-            success, context = set_hierachical(
-                saved_reducers, vector, reducer, False)
+            for vector in vectors:
+                # We set hierarchically for reducers, because
+                # we allow reducers to do a semi matching
+                success, context = set_hierarchical(
+                    saved_reducers, vector, reducer, False)
 
-            if not success:
-                raise ValueError(
-                    f'a reducer{stringify_vector(context)} already exists'
-                )
+                if not success:
+                    raise ValueError(
+                        f'a reducer{stringify_vector(context)} already exists'
+                    )
 
     def connect(
         self,
@@ -102,7 +111,7 @@ class Orchestrator:
         Provider.check(provider)
 
         vector = provider.vector
-        reducer = get_partial_hierachical(self._reducers, vector)
+        reducer = get_partial_hierarchical(self._reducers, vector)
 
         if reducer is None:
             raise KeyError(
@@ -122,7 +131,7 @@ class Orchestrator:
         self._providers[vector] = provider
 
         dispatch = partial(self.dispatch, vector)
-        provider.when_update(dispatch)  # type: ignore
+        provider.when_update(dispatch)
 
         return self
 
@@ -144,7 +153,7 @@ class Orchestrator:
                 raise KeyError(f'a provider{vector_str} must be defined before subscribing to {vector_str}')  # noqa:E501
 
             if vector not in self._subscribed:
-                consumers = []
+                consumers: List[ConsumerSentinel] = []
                 self._subscribed[vector] = consumers
             else:
                 consumers = self._subscribed[vector]
@@ -181,7 +190,7 @@ class Orchestrator:
         if symbol not in self._added:
             return
 
-        reducer = get_partial_hierachical(self._reducers, vector)
+        reducer = get_partial_hierarchical(self._reducers, vector)
 
         if reducer is None:
             raise KeyError(
@@ -189,7 +198,7 @@ class Orchestrator:
             )
 
         store_vector = (symbol, vector)
-        previous = get_hierachical(self._store, store_vector)
+        previous = get_hierarchical(self._store, store_vector)
 
         # We need to try-catch this method,
         # because it won't be raised to the outside and interrupt the program.
@@ -211,7 +220,7 @@ class Orchestrator:
         vector,
         payload
     ) -> None:
-        set_hierachical(self._store, (symbol, vector), payload, True)
+        set_hierarchical(self._store, (symbol, vector), payload, True)
         self._emit(symbol, vector)
 
     def _emit(self, symbol, vector) -> None:
@@ -252,10 +261,33 @@ class Orchestrator:
         if symbol not in self._added:
             self._added.add(symbol)
 
-            self._loop.create_task(self._start_providers(symbol))
+            self._start_symbol_task(symbol)
 
         return self
 
+    def _start_symbol_task(
+        self,
+        symbol: Symbol
+    ) -> None:
+        task = self._create_task(self._start_providers(symbol))
+        self._tasks[symbol] = task
+
+        task.add_done_callback(lambda _: self._remove_symbol_task(symbol))
+
+    def _cancel_symbol_task(
+        self,
+        symbol: Symbol
+    ) -> None:
+        task = self._tasks.get(symbol, None)
+        if task:
+            task.cancel()
+
+    def _remove_symbol_task(
+        self,
+        symbol: Symbol
+    ) -> None:
+        del self._tasks[symbol]
+
     def remove(
         self,
         symbol: Symbol
@@ -272,13 +304,16 @@ class Orchestrator:
             for provider in self._providers.values():
                 provider.remove(symbol)
 
+            self._cancel_symbol_task(symbol)
+
         return self
 
     async def _start_providers(self, symbol: Symbol):
-        await asyncio.wait([
-            self._start_provider(symbol, provider)
+        tasks = [
+            self._create_task(self._start_provider(symbol, provider))
             for provider in self._providers.values()
-        ])
+        ]
+        await asyncio.wait(tasks)
 
     async def _start_provider(
         self,
@@ -301,4 +336,7 @@ class Orchestrator:
             logger.error('give up init symbol "%s"', symbol)
             return
 
+        if payload is None:
+            return
+
         self._dispatch(True, provider.vector, symbol, payload)
