@@ -7,7 +7,7 @@ from typing import (
 )
 
 from .common import (
-    stringify,
+    v_stringify,
     check_vector,
 
     Vector,
@@ -26,7 +26,10 @@ class Provider(ABC):
 
     _dispatch: Optional[Dispatcher]
 
-    __str__ = partialmethod(stringify, 'provider')
+    __str__ = partialmethod(v_stringify, 'provider')  # type: ignore
+    # partialmethod is not supported by mypy,
+    # https://github.com/python/mypy/issues/8619
+    # so we just ignore it
 
     @staticmethod
     def check(provider: Any) -> None:
@@ -69,4 +72,8 @@ class Provider(ABC):
     ) -> None:  # pragma: no cover
         """Sets the receiver to receive update messages
         """
+
+        # So that we can directly use `self.dispatch` in the provider,
+        # However, we could override this method in the provider
+        #   to do something else
         self.dispatch = dispatch
