@@ -26,8 +26,12 @@ def test_no_concurrent_limit_and_outside_event_loop():
     loop = asyncio.new_event_loop()
 
     consumer = SimpleConsumer4()
-    provider = SimpleProvider().go()
-    provider2 = SimpleProvider2().go()
+    provider = SimpleProvider(
+        has_init_delay=False
+    )
+    provider2 = SimpleProvider2(
+        has_init_delay=False
+    )
 
     orchestrator = Orchestrator(
         [SimpleReducer()],
@@ -191,6 +195,25 @@ async def test_main_with_deferred_init():
     assert consumer.consumed == [2]
 
 
+@pytest.mark.asyncio
+async def test_main_with_remove_immediately():
+    consumer = SimpleConsumer()
+    provider = SimpleProvider(
+        update_immediately=False
+    )
+
+    Orchestrator(
+        [SimpleReducer()]
+    ).connect(
+        provider
+    ).subscribe(
+        consumer
+    ).add(symbol).remove(symbol)
+
+    await asyncio.sleep(1)
+    assert consumer.consumed == []
+
+
 def test_reducer_exists():
     with pytest.raises(
         ValueError,
@@ -274,7 +297,8 @@ def test_subscribe_provider_not_found():
         )
 
 
-def test_dispatch_reducer_not_found():
+@pytest.mark.asyncio
+async def test_dispatch_reducer_not_found():
     with pytest.raises(
         KeyError,
         match='reducer<DataType.KLINE,TimeSpan.DAY> is not found'
@@ -292,8 +316,26 @@ def test_dispatch_in_non_coroutine():
         RuntimeError,
         match='should only be invoked in a coroutine'
     ):
-        Orchestrator([]).add(symbol).dispatch(
+        Orchestrator([]).dispatch(
             vector,
             symbol,
             {}
         )
+
+@pytest.mark.asyncio
+async def test_dispatch_with_provider_init_none_payload():
+    class NoneProvider(SimpleProvider):
+        async def init(self, symbol):
+            return None
+
+    consumer = SimpleConsumer()
+
+    Orchestrator([SimpleReducer()]).connect(
+        NoneProvider().go()
+    ).subscribe(
+        consumer
+    ).add(symbol)
+
+    await asyncio.sleep(1)
+
+    assert not consumer.consumed
